"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var sortDescriptor_1 = require("../models/sortDescriptor");
var sortDirection_1 = require("../enums/sortDirection");
var ts_commons_1 = require("ts-commons");
var class_transformer_1 = require("class-transformer");
var SortHelper = /** @class */ (function () {
    function SortHelper() {
    }
    SortHelper.predicateBySorts = function (obj1, obj2, sorts) {
        var result = 0;
        for (var _i = 0, sorts_1 = sorts; _i < sorts_1.length; _i++) {
            var sort = sorts_1[_i];
            result = this.predicateBySortDescriptorBase(obj1, obj2, sort);
            if (result !== 0) {
                break;
            }
        }
        return result;
    };
    SortHelper.predicateBySortDescriptorBase = function (obj1, obj2, sort) {
        var result = 0;
        if (sort instanceof sortDescriptor_1.SortDescriptor) {
            result = this.predicateBySortDescriptor(obj1, obj2, sort);
        }
        else {
            result = 0;
        }
        return result;
    };
    SortHelper.predicateBySortDescriptor = function (obj1, obj2, sortDescriptor) {
        var propertyPath = sortDescriptor.propertyPath;
        var direction = sortDescriptor.direction;
        var propValue1 = this.getDefaultValue(direction, obj1[propertyPath]);
        var propValue2 = this.getDefaultValue(direction, obj2[propertyPath]);
        var result = 0;
        switch (direction) {
            case sortDirection_1.SortDirection.ASC:
            case sortDirection_1.SortDirection.ASC_NULL_FIRST:
            case sortDirection_1.SortDirection.ASC_NULL_LAST:
                if (propValue1 === propValue2) {
                    result = 0;
                }
                else if (propValue1 > propValue2) {
                    result = 1;
                }
                else {
                    result = -1;
                }
                break;
            case sortDirection_1.SortDirection.DESC:
            case sortDirection_1.SortDirection.DESC_NULL_FIRST:
            case sortDirection_1.SortDirection.DESC_NULL_LAST:
                if (propValue1 === propValue2) {
                    result = 0;
                }
                else if (propValue1 < propValue2) {
                    result = 1;
                }
                else {
                    result = -1;
                }
                break;
        }
        return result;
    };
    SortHelper.getRealSorts = function (sorts) {
        if (ts_commons_1.ArrayUtils.isEmpty(sorts)) {
            return [];
        }
        var result = [];
        for (var _i = 0, _a = sorts || []; _i < _a.length; _i++) {
            var filterBase = _a[_i];
            var filterJson = class_transformer_1.serialize(filterBase);
            switch (filterBase.type) {
                case "SortDescriptor":
                    result.push(new sortDescriptor_1.SortDescriptor().fromJSON(filterJson));
                    break;
            }
        }
        return result;
    };
    SortHelper.getDefaultValue = function (sortDirection, value) {
        if (sortDirection == sortDirection_1.SortDirection.ASC_NULL_FIRST &&
            ts_commons_1.ObjectUtils.isNullOrUndefined(value)) {
            return ts_commons_1.NumberUtils.MIN_SAFE_INTEGER;
        }
        if (sortDirection == sortDirection_1.SortDirection.DESC_NULL_FIRST &&
            ts_commons_1.ObjectUtils.isNullOrUndefined(value)) {
            return ts_commons_1.NumberUtils.MAX_SAFE_INTEGER;
        }
        if (sortDirection == sortDirection_1.SortDirection.ASC_NULL_LAST &&
            ts_commons_1.ObjectUtils.isNullOrUndefined(value)) {
            return ts_commons_1.NumberUtils.MAX_SAFE_INTEGER;
        }
        if (sortDirection == sortDirection_1.SortDirection.DESC_NULL_LAST &&
            ts_commons_1.ObjectUtils.isNullOrUndefined(value)) {
            return ts_commons_1.NumberUtils.MIN_SAFE_INTEGER;
        }
        if (ts_commons_1.ObjectUtils.isNullOrUndefined(value)) {
            // default return min value same as mysql
            return ts_commons_1.NumberUtils.MIN_SAFE_INTEGER;
        }
        else {
            return value;
        }
    };
    return SortHelper;
}());
exports.SortHelper = SortHelper;
//# sourceMappingURL=sortHelper.js.map