"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var class_transformer_1 = require("class-transformer");
var filterHelper_1 = require("../helpers/filterHelper");
var filterDescriptor_1 = require("./filterDescriptor");
var filterGroupDescriptor_1 = require("./filterGroupDescriptor");
var sortDescriptor_1 = require("./sortDescriptor");
var ts_commons_1 = require("ts-commons");
var filterCondition_1 = require("../enums/filterCondition");
var queryProvider_1 = require("../providers/queryProvider");
var sortHelper_1 = require("../helpers/sortHelper");
var DynamicQuery = /** @class */ (function () {
    function DynamicQuery() {
        this.filters = [];
        this.sorts = [];
        this.selectedProperties = [];
    }
    DynamicQuery.createQuery = function (type) {
        var instance = new DynamicQuery();
        instance.type = type;
        return instance;
    };
    DynamicQuery.prototype.addFilters = function (filters) {
        this.filters = this.filters.concat(filters);
        return this;
    };
    DynamicQuery.prototype.addFilterDescriptor = function (option) {
        var filter = new filterDescriptor_1.FilterDescriptor(option);
        this.filters.push(filter);
        return this;
    };
    DynamicQuery.prototype.addFilterGroupDescriptor = function (option) {
        var filterGroupDescriptor = new filterGroupDescriptor_1.FilterGroupDescriptor();
        filterGroupDescriptor.condition = ts_commons_1.ObjectUtils.isNullOrUndefined(option.condition)
            ? filterCondition_1.FilterCondition.AND
            : option.condition;
        for (var _i = 0, _a = option.options; _i < _a.length; _i++) {
            var subOption = _a[_i];
            filterGroupDescriptor.filters.push(new filterDescriptor_1.FilterDescriptor(subOption));
        }
        this.filters.push(filterGroupDescriptor);
        return this;
    };
    DynamicQuery.prototype.addSorts = function (sorts) {
        this.sorts = this.sorts.concat(sorts);
        return this;
    };
    DynamicQuery.prototype.addSortDescriptor = function (sortOption) {
        var sort = new sortDescriptor_1.SortDescriptor(sortOption);
        this.sorts.push(sort);
        return this;
    };
    DynamicQuery.prototype.selectProperty = function (property) {
        var propertyStr = property.toString();
        this.selectedProperties.push(propertyStr);
        return this;
    };
    DynamicQuery.prototype.selectProperties = function () {
        var properties = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            properties[_i] = arguments[_i];
        }
        var propertyStrs = properties.map(function (x) { return x.toString(); });
        this.selectedProperties = this.selectedProperties.concat(propertyStrs);
        return this;
    };
    DynamicQuery.prototype.query = function (datas) {
        return queryProvider_1.QueryProvider.query(datas, this);
    };
    DynamicQuery.prototype.toJSON = function () {
        return class_transformer_1.serialize(this);
    };
    DynamicQuery.prototype.fromJSON = function (json) {
        var obj = class_transformer_1.deserialize(DynamicQuery, json);
        this.filters = filterHelper_1.FilterHelper.getRealFilters(obj.filters);
        this.sorts = sortHelper_1.SortHelper.getRealSorts(obj.sorts);
        return this;
    };
    return DynamicQuery;
}());
exports.DynamicQuery = DynamicQuery;
//# sourceMappingURL=dynamicQuery.js.map